class TestInitDomain(TestBase):
	"""
	Test that pid 1 (init) is running in the appropriate domain
	"""
	class ErrorGetfileconFailed(ErrorBase):
		def __str__(self):
			return "Could not read the domain of PID 1."

	class ErrorInitBadlyLabeled(ErrorBase):
		def __str__(self):
			return "The init process (PID 1) is running in an incorrect domain."

	@staticmethod
	def test():
		from subprocess import Popen, PIPE

		contextok = False

		pipe = Popen("getfilecon /proc/1", shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True, universal_newlines=True)
		pipe.stdin.close()

		for line in pipe.stdout.readlines():
			line = line.rstrip()
			if line == "": continue
			if line.endswith(":system_r:init_t") \
				or line.find(":system_r:init_t:") >= 0:
				contextok = True
			else:
				print("..%s.." % line)
		pipe.stdout.close()

		for line in pipe.stderr.readlines():
			if line.find("failed") >= 0:
				contextok = "failed"
		pipe.stderr.close()

		if contextok == "failed":
			return [TestInitDomain.ErrorGetfileconFailed()]
		if not contextok:
			return [TestInitDomain.ErrorInitBadlyLabeled()]
		return []
register_test(TestInitDomain)



class TestNoKernelT(TestBase):
	"""
	Test for processes running in system_u:system_r:kernel_t
	This type should only be used by kernel processes, which are detected by
	not having any maps.
	"""
	# Not sure if this test actually helps when the init test succeeded.
	# But it doesn't harm either

	class ErrorBadKernelProcesses(ErrorBase):
		def __init__(self, badprocs):
			self.badprocs = badprocs
		def __str__(self):
			return "There were %d processes found running in the kernel domain." \
				% len(self.badprocs)

	@staticmethod
	def test():
		from subprocess import Popen, PIPE

		badprocs = []

		pipe = Popen("getfilecon /proc/[0-9]*", shell=True, stdin=PIPE, stdout=PIPE, close_fds=True, universal_newlines=True)
		pipe.stdin.close()
		for line in pipe.stdout.readlines():
			(dir, context) = line.split()
			components = dir.split("/")
			pid = components[-1]
			if context.find("system_r:kernel_t") >= 0:
				badproc = False
				file = open("/proc/%s/maps" % pid)
				if file.readlines():
					badproc = True
				file.close()
				if badproc:
					badprocs.append(pid)
		pipe.stdout.close()
		if len(badprocs) > 0:
			return [TestNoKernelT.ErrorBadKernelProcesses(badprocs)]
		return []
register_test(TestNoKernelT)



class TestFSCKFix(TestBase):
	"""
	Suggest to use FSCKFIX in /etc/default/rcS
	"""
	class ErrorFSCKFix(ErrorBase):
		def __str__(self):
			return "FSCKFIX is not enabled - not serious, but could prevent system from booting..."

	@staticmethod
	def test():
		import os, re
		r = re.compile(r'^\s*FSCKFIX=.*yes')

		if os.access("/etc/default/rcS", os.F_OK):
			fsckfix = False
			f = open("/etc/default/rcS","r")
			for line in f.readlines():
				if r.match(line):
					fsckfix = True
			f.close()
			if not fsckfix:
				return [TestFSCKFix.ErrorFSCKFix()]
		else:
			raise IOError("/etc/default/rcS not found, is this Debian?")
		return []
register_test(TestFSCKFix)



class TestPAMConfig(TestBase):
	"""
	Verify that pam is setup for SELinux
	"""
	class ErrorPAMSSH(ErrorBase):
		def __str__(self):
			return "/etc/pam.d/sshd is not SELinux enabled"

	class ErrorPAMLogin(ErrorBase):
		def __str__(self):
			return "/etc/pam.d/login is not SELinux enabled"

	@staticmethod
	def test():
		import os, re
		r = re.compile(r'^\s*session\s+(required|\[success=ok\s+ignore=ignore\s+module_unknown=ignore\s+default=bad\])\s+pam_selinux.so')
		result = []

		if os.access("/etc/pam.d/sshd", os.F_OK):
			selinuxon = False
			f = open("/etc/pam.d/sshd","r")
			for line in f.readlines():
				if r.match(line):
					selinuxon = True
			f.close()
			if not selinuxon:
				result.append(TestPAMConfig.ErrorPAMSSH())

		if os.access("/etc/pam.d/login", os.F_OK):
			selinuxon = False
			f = open("/etc/pam.d/login","r")
			for line in f.readlines():
				if r.match(line):
					selinuxon = True
			f.close()
			if not selinuxon:
				result.append(TestPAMConfig.ErrorPAMLogin())
		return result
register_test(TestPAMConfig)



class TestOldTTYs(TestBase):
	"""
	Check for old-style ttys that are not supported by the policies.
	These are disabled in the default kernel shipped by Debian since
	Squeeze.
	"""

	class ErrorOldTTYs(ErrorBase):
		def __init__(self, oldttys):
			self.oldttys = oldttys
		def __str__(self):
			return "Old style ttys were found."
		def fixable(self):
			return True
		def fix(self):
			success = True
			for nam in self.oldttys:
				if not os.unlink(nam):
					success = False
			return success

	@staticmethod
	def test():
		import glob

		oldttys = glob.glob("/dev/[tp]ty[abcdepqrstuvwxyz][0-9a-f]")
		if len(oldttys) > 0:
			return [TestOldTTYs.ErrorOldTTYs(oldttys)]
		return []
register_test(TestOldTTYs)



class TestSlashSELinux(TestBase):
	"""
	Verify that /sys/fs/selinux or /selinux is present
	"""
	class ErrorNoSlashSELinux(ErrorBase):
		def __str__(self):
			return "The directories /sys/fs/selinux and /selinux are missing."
		def fixable(self):
			return True
		def fix(self):
			import os
			return os.mkdir("/selinux")

	@staticmethod
	def test():
		import os
		if not os.access("/sys/fs/selinux",os.F_OK):
			if not os.access("/selinux",os.F_OK):
				return [TestSlashSELinux.ErrorNoSlashSELinux()]
		return []
register_test(TestSlashSELinux)
